function explore(o)
% explore (exploratory workshop companion code)
% by Christian Himpe, 2015 ( http://gramian.de )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    if(exist('emgr')~=2)
        error('emgr not found! Get emgr at: http://gramian.de');
    else
        global ODE;
        fprintf('emgr (version: %g)\n',emgr('version'));
    end

    if(nargin==0), o = 0; end;

%% Setup
    J = 1;
    N = 64;
    O = J;
    T = [0,0.01,1.0];
    L = round(T(3)-T(1))/T(2);
    U = [ones(J,1)./T(2),zeros(J,L-1)];
    X = zeros(N,1);

    rand('seed',1009);
    A = sprand(N,N,0.15);
    A(1:N+1:end) = -N;
    ISTABLE = all(real(eig(A))<0)
    B = [30.0;zeros(N-1,1)];
    C = rand(O,N);
    Q = [0.5*ones(N,1),ones(N,1)];

    NON = @(x,u,p) A*tanh(p.*x) + B*u;
    OUT = @(x,u,p) C*x;

    %Y = ODE(NON,OUT,T,X,U,rand(N,1)); figure; plot(T(1):T(2):T(3),Y); return;

%% Offline
    tic;
    WJ = emgr(NON,OUT,[J,N,O],T,'j',Q,[1,0,0,0,0,0,0,0,0,0],1,0,B);
    [UU,D,VV] = svd(WJ{1});
    [PP,D,QQ] = svd(WJ{2});
    OFFLINE = toc

%% Online
    P = 0.5*rand(N,10)+0.5;
    S = size(P,2);
    n = 4;
    err = zeros(N/n,N/n,3,S);

    for I=1:S
        Y = ODE(NON,OUT,T,X,U,P(:,I));
        a = 1;
        for KX=1:4:N
            uu = UU(:,1:KX);
            vv = uu';
            b = 1;
            for KP=1:4:N
                pp = PP(:,1:KP);
                qq = pp';
                p = qq*P(:,I);
                x = vv*X;
                non = @(x,u,p) vv*NON(uu*x,u,p);
                out = @(x,u,p) OUT(uu*x,u,p);
                y = ODE(non,out,T,x,U,pp*p);
                err(a,b,1,I) = norm(Y(:)-y(:),1)/norm(Y(:),1);
                err(a,b,2,I) = norm(Y(:)-y(:),2)/norm(Y(:),2);
                err(a,b,3,I) = norm(Y(:)-y(:),Inf)/norm(Y(:),Inf);
                b = b + 1;
            end;
            a = a + 1;
        end;
    end;

    err = sqrt(sum(err.^2,4));

    m  = 1e-6;
    %surfplot(err,1,n,N,m,o);
    surfplot(err,2,n,N,m,o);
    %surfplot(err,3,n,N,m,o);

end

%% ======== Surfplot ========
function surfplot(e,i,n,N,m,o)

    figure();
    h = surf(e(:,:,i));
    xlim([1,size(e(:,:,i),1)]);
    ylim([1,size(e(:,:,i),2)]);
    zlim([m,1]);
    set(gca,'ZScale','log');
    set(gca,'XTick',1:2:16,'XTickLabel',1:2*n:N);
    set(gca,'YTick',1:2:16,'YTickLabel',1:2*n:N);
    ylabel('State Dimension')
    xlabel('Parameter Dimension');
    colormap(antijet);
    set(h,'CData',log10(get(h,'CData')))
    view(135,30);

    if(o==1), print('-dpdf',[mfilename(),num2str(i),'.pdf']); end;
end

%% ======== Colormap ========
function m = antijet(n)
% antijet colormap
% by Christian Himpe 2014-2015
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    if(nargin<1 || isempty(n)), n = 256; end;
    L = linspace(0,1,n);

    R = -0.5*sin( L*(1.37*pi)+0.13*pi )+0.5;
    G = -0.4*cos( L*(1.5*pi) )+0.4;
    B = 0.3*sin( L*(2.11*pi) )+0.3;

    m = [R;G;B]';
end
